/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkRubyAppInit.cxx,v $

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/


/* Minimal main program -- everything is loaded from the library */

#include "vtkRuby.h"

#ifdef VTK_COMPILED_USING_MPI
# include <mpi.h>
# include "vtkMPIController.h"
#endif // VTK_COMPILED_USING_MPI

#include "vtkVersion.h"
#include "Wrapping/Ruby/vtkRubyAppInitConfigure.h"

#if defined(CMAKE_INTDIR)
# define VTK_RUBY_LIBRARY_DIR VTK_RUBY_LIBRARY_DIR_BUILD "/" CMAKE_INTDIR
#else
# define VTK_RUBY_LIBRARY_DIR VTK_RUBY_LIBRARY_DIR_BUILD
#endif

#include <sys/stat.h>

/*
 * Make sure all the kits register their classes with vtkInstantiator.
 */
#include "vtkCommonInstantiator.h"
#include "vtkFilteringInstantiator.h"
#include "vtkIOInstantiator.h"
#include "vtkImagingInstantiator.h"
#include "vtkGraphicsInstantiator.h"

#ifdef VTK_USE_RENDERING
#include "vtkRenderingInstantiator.h"
#include "vtkVolumeRenderingInstantiator.h"
#include "vtkHybridInstantiator.h"
#endif

#ifdef VTK_USE_PARALLEL
#include "vtkParallelInstantiator.h"
#endif

#include <vtkstd/string>
#include <vtksys/SystemTools.hxx>

#ifdef VTK_COMPILED_USING_MPI
class vtkMPICleanup {
public:
  vtkMPICleanup()
    {
      this->Controller = 0;
    }
  void Initialize(int* argc, char ***argv)
    {
      MPI_Init(argc, argv);
      this->Controller = vtkMPIController::New();
      this->Controller->Initialize(argc, argv, 1);
      vtkMultiProcessController::SetGlobalController(this->Controller);
    }
  ~vtkMPICleanup()
    {
      if ( this->Controller )
        {
        this->Controller->Finalize();
        this->Controller->Delete();
        }
    }
private:
  vtkMPIController *Controller;
};

static vtkMPICleanup VTKMPICleanup;

#endif // VTK_COMPILED_USING_MPI


static void vtkRubyAppInitEnableMSVCDebugHook();
static void vtkRubyAppInitPrependPath(void);

/* The maximum length of a file name.  */
#if defined(PATH_MAX)
# define VTK_RUBY_MAXPATH PATH_MAX
#elif defined(MAXPATHLEN)
# define VTK_RUBY_MAXPATH MAXPATHLEN
#else
# define VTK_RUBY_MAXPATH 16384
#endif

/* Ruby major.minor version string.  */
#define VTK_RUBY_TO_STRING(x) VTK_RUBY_TO_STRING0(x)
#define VTK_RUBY_TO_STRING0(x) VTK_RUBY_TO_STRING1(x)
#define VTK_RUBY_TO_STRING1(x) #x
#define VTK_RUBY_VERSION VTK_RUBY_TO_STRING(PY_MAJOR_VERSION.PY_MINOR_VERSION)

int main(int argc, char **argv)
{
  vtkRubyAppInitEnableMSVCDebugHook();

#ifdef VTK_COMPILED_USING_MPI
  VTKMPICleanup.Initialize(&argc, &argv);
#endif // VTK_COMPILED_USING_MPI

  int displayVersion = 0;
  if ( argc > 1 )
    {
    int cc;
    for ( cc = 1; cc < argc; cc ++ )
      {
      if ( strcmp(argv[cc], "-V") == 0 )
        {
        displayVersion = 1;
        break;
        }
      }
    }
  else
    {
    displayVersion = 1;
    }
  if ( displayVersion )
    {
    cout << vtkVersion::GetVTKSourceVersion() << endl;
    }

  // The following code will hack in the path for running VTK/Ruby
  // from the build tree. Do not try this at home. We are
  // professionals.

  // Initialize interpreter.
  ruby_init();
  ruby_options(argc, argv);

  vtkRubyAppInitPrependPath();

  // Ok, all done, now enter ruby main.
  ruby_run();
  return 0;
}

// For a DEBUG build on MSVC, add a hook to prevent error dialogs when
// being run from DART.
#if defined(_MSC_VER) && defined(_DEBUG)
# include <crtdbg.h>
static int vtkRubyAppInitDebugReport(int, char* message, int*)
{
  fprintf(stderr, message);
  exit(1);
}
void vtkRubyAppInitEnableMSVCDebugHook()
{
  if(getenv("DART_TEST_FROM_DART"))
    {
    _CrtSetReportHook(vtkRubyAppInitDebugReport);
    }
}
#else
void vtkRubyAppInitEnableMSVCDebugHook()
{
}
#endif

//----------------------------------------------------------------------------
static void vtkRubyAppInitPrependRubyPath(const char* dir)
{
  // Convert slashes for this platform.
  vtkstd::string out_dir = dir;
#if defined(_WIN32) && !defined(__CYGWIN__)
  for(vtkstd::string::size_type i = 0; i < out_dir.length(); ++i)
    {
    if(out_dir[i] == '/')
      {
      out_dir[i] = '\\';
      }
    }
#endif

  // Append the path to the $LOAD_PATH
  ruby_incpush(dir);
}

//----------------------------------------------------------------------------
static void vtkRubyAppInitPrependPath()
{
  vtkRubyAppInitPrependRubyPath(VTK_RUBY_LIBRARY_DIR);
  vtkRubyAppInitPrependRubyPath(VTK_RUBY_SCRIPT_DIR_BUILD);

}
